@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,18 +22,21 @@
 Create cleaners from CleanerML (markup language)
 """
 
-import bleachbit
-from bleachbit.Action import ActionProvider
-from bleachbit import _
-from bleachbit.General import boolstr_to_bool, getText
-from bleachbit.FileUtilities import expand_glob_join, listdir
-from bleachbit import Cleaner
-
+# standard library
 import logging
 import os
+import stat
 import sys
 import xml.dom.minidom
 
+# local import
+import bleachbit
+from bleachbit.Action import ActionProvider
+from bleachbit.FileUtilities import expand_glob_join, listdir
+from bleachbit.General import boolstr_to_bool, getText
+from bleachbit.Language import get_text as _
+from bleachbit import Cleaner
+
 logger = logging.getLogger(__name__)
 
 
@@ -78,7 +81,12 @@ class CleanerML:
         else:
             self.xlate_mode = True
 
-        dom = xml.dom.minidom.parse(pathname)
+        try:
+            dom = xml.dom.minidom.parse(pathname)
+        except xml.parsers.expat.ExpatError as e:
+            logger.error(
+                "Error parsing CleanerML file %s with error %s", pathname, e)
+            return
 
         self.handle_cleaner(dom.getElementsByTagName('cleaner')[0])
 
@@ -100,7 +108,7 @@ class CleanerML:
         # Define the current operating system.
         if platform == 'darwin':
             current_os = ('darwin', 'bsd', 'unix')
-        elif platform.startswith('linux'):
+        elif platform == 'linux':
             current_os = ('linux', 'unix')
         elif platform.startswith('openbsd'):
             current_os = ('bsd', 'openbsd', 'unix')
@@ -109,9 +117,9 @@ class CleanerML:
         elif platform.startswith('freebsd'):
             current_os = ('bsd', 'freebsd', 'unix')
         elif platform == 'win32':
-            current_os = ('windows')
+            current_os = ('windows',)
         else:
-            raise RuntimeError('Unknown operating system: %s ' % sys.platform)
+            raise RuntimeError(f'Unknown operating system: {sys.platform}')
         # Compare current OS against required OS.
         return os_str in current_os
 
@@ -129,11 +137,11 @@ class CleanerML:
         for option in cleaner.getElementsByTagName('option'):
             try:
                 self.handle_cleaner_option(option)
-            except:
+            except Exception:
                 exc_msg = _(
                     "Error in handle_cleaner_option() for cleaner id = {cleaner_id}, option XML={option_xml}")
                 logger.exception(exc_msg.format(
-                    cleaner_id=exc_dict, option_xml=option.toxml()))
+                    cleaner_id=self.cleaner.id, option_xml=option.toxml()))
         self.handle_cleaner_running(cleaner.getElementsByTagName('running'))
         self.handle_localizations(
             cleaner.getElementsByTagName('localizations'))
@@ -159,7 +167,8 @@ class CleanerML:
                 continue
             detection_type = running.getAttribute('type')
             value = getText(running.childNodes)
-            self.cleaner.add_running(detection_type, value)
+            same_user = running.getAttribute('same_user') or False
+            self.cleaner.add_running(detection_type, value, same_user)
 
     def handle_cleaner_option(self, option):
         """<option> element"""
@@ -212,13 +221,14 @@ class CleanerML:
             if actionplugin.action_key == command:
                 provider = actionplugin(action_node, self.vars)
         if provider is None:
-            raise RuntimeError("Invalid command '%s'" % command)
+            raise RuntimeError(f"Invalid command '{command}'")
         self.cleaner.add_action(self.option_id, provider)
 
     def handle_localizations(self, localization_nodes):
         """<localizations> element under <cleaner>"""
         if not 'posix' == os.name:
             return
+        # pylint: disable=import-outside-toplevel
         from bleachbit import Unix
         for localization_node in localization_nodes:
             for child_node in localization_node.childNodes:
@@ -259,14 +269,15 @@ def list_cleanerml_files(local_only=False):
     """List CleanerML files"""
     cleanerdirs = (bleachbit.personal_cleaners_dir, )
     if bleachbit.local_cleaners_dir:
-        # If the application is installed, locale_cleaners_dir is None
-        cleanerdirs = (bleachbit.local_cleaners_dir, )
+        # If the application is installed, locale_cleaners_dir is None.
+        # If portable mode, local_cleaners_dir is under the directory of
+        # `bleachbit.py`.
+        cleanerdirs += (bleachbit.local_cleaners_dir, )
     if not local_only and bleachbit.system_cleaners_dir:
         cleanerdirs += (bleachbit.system_cleaners_dir, )
     for pathname in listdir(cleanerdirs):
         if not pathname.lower().endswith('.xml'):
             continue
-        import stat
         st = os.stat(pathname)
         if sys.platform != 'win32' and stat.S_IMODE(st[stat.ST_MODE]) & 2:
             # TRANSLATORS: When BleachBit detects the file permissions are
@@ -290,7 +301,7 @@ def load_cleaners(cb_progress=lambda x: None):
     for pathname in cleanerml_files:
         try:
             xmlcleaner = CleanerML(pathname)
-        except:
+        except Exception:
             logger.exception(_("Error reading cleaner: %s"), pathname)
             continue
         cleaner = xmlcleaner.get_cleaner()
@@ -311,34 +322,31 @@ def pot_fragment(msgid, pathname, translators=None):
     """Create a string fragment for generating .pot files"""
     msgid = msgid.replace('"', '\\"')  # escape quotation mark
     if translators:
-        translators = "#. %s\n" % translators
+        translators = f"#. {translators}\n"
     else:
         translators = ""
-    ret = '''%s#: %s
-msgid "%s"
+    ret = f'''{translators}#: {pathname}
+msgid "{msgid}"
 msgstr ""
 
-''' % (translators, pathname, msgid)
+'''
     return ret
 
 
 def create_pot():
     """Create a .pot for translation using gettext"""
 
-    f = open('../po/cleanerml.pot', 'w')
-
-    for pathname in listdir('../cleaners'):
-        if not pathname.lower().endswith(".xml"):
-            continue
-        strings = []
-        try:
-            CleanerML(pathname,
-                      lambda newstr, translators=None:
-                      strings.append([newstr, translators]))
-        except:
-            logger.exception(_("Error reading cleaner: %s"), pathname)
-            continue
-        for (string, translators) in strings:
-            f.write(pot_fragment(string, pathname, translators))
-
-    f.close()
+    with open('../po/cleanerml.pot', 'w', encoding='utf-8') as f:
+        for pathname in listdir('../cleaners'):
+            if not pathname.lower().endswith(".xml"):
+                continue
+            strings = []
+            try:
+                CleanerML(pathname,
+                          lambda newstr, translators=None, current_strings=strings:
+                          current_strings.append([newstr, translators]))
+            except Exception:
+                logger.exception(_("Error reading cleaner: %s"), pathname)
+                continue
+            for (string, translators) in strings:
+                f.write(pot_fragment(string, pathname, translators))
