@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,7 +24,7 @@ File-related utilities
 """
 
 import bleachbit
-from bleachbit import _
+from bleachbit.Language import get_text as _
 
 import atexit
 import errno
@@ -57,10 +57,12 @@ if 'posix' == os.name:
     pywinerror = WindowsError
 
 try:
+    # FIXME: replace scandir.walk() with os.scandir()
+    # Preserve behavior added in 25e694.
+    # Test that os.walk() behaves the same on Windows.
     from scandir import walk
     if 'nt' == os.name:
         import scandir
-        import bleachbit.Windows
 
         class _Win32DirEntryPython(scandir.Win32DirEntryPython):
             def is_symlink(self):
@@ -69,10 +71,7 @@ try:
         scandir.scandir = scandir.scandir_python
         scandir.DirEntry = scandir.Win32DirEntryPython = _Win32DirEntryPython
 except ImportError:
-    if sys.version_info < (3, 5, 0):
-        # Python 3.5 incorporated scandir
-        logger.warning(
-            'scandir is not available, so falling back to slower os.walk()')
+    # Since Python 3.5, os.walk() calls os.scandir().
     from os import walk
 
 
@@ -81,39 +80,54 @@ def open_files_linux():
 
 
 def get_filesystem_type(path):
-    """
-    * Get file system type from the given path
-    * return value: The tuple of (file_system_type, device_name)
-    *               @ file_system_type: vfat, ntfs, etc
-    *               @ device_name:      C://, D://, etc
+    """Get file system type from the given path
+
+    path: directory path
+
+    Return value:
+    A tuple of (file_system_type, device_name)
+    file_system_type: vfat, ntfs, etc.
+    device_name: C:, D:, etc.
+
+    File system types seen
+    * On Linux: btrfs,ext4, vfat, squashfs
+    * On Windows: NTFS, FAT32, CDFS
     """
     try:
         import psutil
+        from pathlib import Path
     except ImportError:
-        logger.warning('To get the file system type from the given path, you need to install psutil package')
+        logger.warning(
+            'To get the file system type from the given path, you need to install psutil package')
         return ("unknown", "none")
 
+    path_obj = Path(path)
+    if os.name == 'nt':
+        if len(path) == 2 and path[1] == ':':
+            path_obj = Path(path + '\\')
+
+    # Get all partitions with Path objects as keys
     partitions = {}
     for partition in psutil.disk_partitions():
-        partitions[partition.mountpoint] = (partition.fstype, partition.device)
-        
-    if path in partitions:
-        return partitions[path]
+        mount_path = Path(partition.mountpoint)
+        partitions[mount_path] = (partition.fstype, partition.device)
 
-    splitpath = path.split(os.sep)
-    for i in range(0, len(splitpath)-1):
-        path = os.sep.join(splitpath[:i]) + os.sep
-        if path in partitions:
-            return partitions[path]
-            
-        path = os.sep.join(splitpath[:i])
-        if path in partitions:
-            return partitions[path]
+    # Exact match
+    for mount_path in partitions:
+        if path_obj == mount_path:
+            return partitions[mount_path]
+
+    # Try parent paths
+    current = path_obj
+    while current.parent != current:  # Stop at root
+        current = current.parent
+        for mount_path in partitions:
+            if current == mount_path:
+                return partitions[mount_path]
 
     return ("unknown", "none")
 
 
-
 def open_files_lsof(run_lsof=None):
     if run_lsof is None:
         def run_lsof():
@@ -124,7 +138,7 @@ def open_files_lsof(run_lsof=None):
 
 
 def open_files():
-    if sys.platform.startswith('linux'):
+    if sys.platform == 'linux':
         files = open_files_linux()
     elif 'darwin' == sys.platform or sys.platform.startswith('freebsd'):
         files = open_files_lsof()
@@ -137,6 +151,10 @@ def open_files():
             # happens, for example, when link points to
             # '/etc/password\x00 (deleted)'
             continue
+        except PermissionError:
+            # /proc/###/fd/0 with systemd
+            # https://github.com/bleachbit/bleachbit/issues/1515
+            continue
         else:
             yield target
 
@@ -191,10 +209,10 @@ def bytes_to_human(bytes_i):
         prefixes = ['', 'k', 'M', 'G', 'T', 'P']
         base = 1000.0
 
-    assert(isinstance(bytes_i, int))
+    assert (isinstance(bytes_i, int))
 
     if 0 == bytes_i:
-        return "0"
+        return '0B'
 
     if bytes_i >= base ** 3:
         decimals = 2
@@ -217,8 +235,7 @@ def children_in_directory(top, list_directories=False):
     """Iterate files and, optionally, subdirectories in directory"""
     if type(top) is tuple:
         for top_ in top:
-            for pathname in children_in_directory(top_, list_directories):
-                yield pathname
+            yield from children_in_directory(top_, list_directories)
         return
     for (dirpath, dirnames, filenames) in walk(top, topdown=False):
         if list_directories:
@@ -229,7 +246,10 @@ def children_in_directory(top, list_directories=False):
 
 
 def clean_ini(path, section, parameter):
-    """Delete sections and parameters (aka option) in the file"""
+    """Delete sections and parameters (aka option) in the file
+
+    Comments are not preserved.
+    """
 
     def write(parser, ini_file):
         """
@@ -293,7 +313,7 @@ def clean_json(path, target):
     targets = target.split('/')
 
     # read file to parser
-    with open(path, 'r', encoding='utf-8') as f:
+    with open(path, 'r', encoding='utf-8-sig') as f:
         js = json.load(f)
 
     # change file
@@ -308,7 +328,7 @@ def clean_json(path, target):
         elif new_target in pos:
             # delete terminal target
             changed = True
-            del(pos[new_target])
+            del (pos[new_target])
         else:
             # target not found
             break
@@ -414,15 +434,6 @@ def detect_encoding(fn):
         return None
 
     with open(fn, 'rb') as f:
-        if not hasattr(chardet, 'universaldetector'):
-            # This method works on Ubuntu 16.04 with an older version of the module.
-            rawdata = f.read()
-            det = chardet.detect(rawdata)
-            if det['confidence'] > 0.5:
-                return det['encoding']
-            return None
-
-        # This method is faster, but it requires a newer version of the module.
         detector = chardet.universaldetector.UniversalDetector()
         for line in f.readlines():
             detector.feed(line)
@@ -457,15 +468,28 @@ def exe_exists(pathname):
 
 
 def execute_sqlite3(path, cmds):
-    """Execute 'cmds' on SQLite database 'path'"""
+    """Execute SQL commands on SQLite database
+
+    Args:
+        path (str): Path to the SQLite database file
+        cmds (str): SQL commands to execute, separated by semicolons
+
+    Raises:
+        sqlite3.OperationalError: If there's an error executing the SQL commands
+        sqlite3.DatabaseError: If there's a database-related error
+
+    Returns:
+        None
+    """
     import sqlite3
-    import contextlib
-    with contextlib.closing(sqlite3.connect(path)) as conn:
+    from bleachbit.Options import options
+    assert isinstance(path, str)
+    assert isinstance(cmds, str)
+    with sqlite3.connect(path) as conn:
         cursor = conn.cursor()
 
         # overwrites deleted content with zeros
         # https://www.sqlite.org/pragma.html#pragma_secure_delete
-        from bleachbit.Options import options
         if options.get('shred'):
             cursor.execute('PRAGMA secure_delete=ON')
 
@@ -485,22 +509,23 @@ def execute_sqlite3(path, cmds):
                     '%s: %s' % (exc, path))
 
         cursor.close()
-        conn.commit()
+        from bleachbit.General import gc_collect
+        gc_collect()
 
 
 def expand_glob_join(pathname1, pathname2):
     """Join pathname1 and pathname1, expand pathname, glob, and return as list"""
-    ret = []
     pathname3 = os.path.expanduser(os.path.expandvars(
         os.path.join(pathname1, pathname2)))
-    for pathname4 in glob.iglob(pathname3):
-        ret.append(pathname4)
+    ret = [pathname4 for pathname4 in glob.iglob(pathname3)]
     return ret
 
 
 def extended_path(path):
     """If applicable, return the extended Windows pathname"""
-    if 'nt' == os.name:
+    # Do not extend the Sysnative paths because on some systems there are problems with path resolution,
+    # for example: https://github.com/bleachbit/bleachbit/issues/1574.
+    if 'nt' == os.name and 'Sysnative' not in path.split(os.sep):
         if path.startswith(r'\\?'):
             return path
         if path.startswith(r'\\'):
@@ -522,16 +547,8 @@ def extended_path_undo(path):
 def free_space(pathname):
     """Return free space in bytes"""
     if 'nt' == os.name:
-        from bleachbit import Windows
-        if Windows.parse_windows_build() >= 6:
-            # This works better with UTF-8 paths.
-            import psutil
-            return psutil.disk_usage(pathname).free
-        else:
-            # This works better with Windows XP but not UTF-8.
-            # Deprecated.
-            _fb, _tb, total_free_bytes = win32file.GetDiskFreeSpaceEx(pathname)
-            return total_free_bytes
+        import psutil
+        return psutil.disk_usage(pathname).free
     mystat = os.statvfs(pathname)
     return mystat.f_bfree * mystat.f_bsize
 
@@ -555,7 +572,12 @@ def getsize(path):
         # try FindFilesW.
         # Also, apply prefix to use extended-length paths to support longer
         # filenames.
-        finddata = win32file.FindFilesW(extended_path(path))
+        try:
+            finddata = win32file.FindFilesW(extended_path(path))
+        except pywinerror as e:
+            if e.winerror == 3:  # 3 = The system cannot find the path specified.
+                raise OSError(errno.ENOENT, e.strerror, path)
+            raise e
         if not finddata:
             # FindFilesW does not work for directories, so fall back to
             # getsize()
@@ -568,9 +590,10 @@ def getsize(path):
 
 def getsizedir(path):
     """Return the size of the contents of a directory"""
-    total_bytes = 0
-    for node in children_in_directory(path, list_directories=False):
-        total_bytes += getsize(node)
+    total_bytes = sum(
+        getsize(node)
+        for node in children_in_directory(path, list_directories=False)
+    )
     return total_bytes
 
 
@@ -578,8 +601,7 @@ def globex(pathname, regex):
     """Yield a list of files with pathname and filter by regex"""
     if type(pathname) is tuple:
         for singleglob in pathname:
-            for path in globex(singleglob, regex):
-                yield path
+            yield from globex(singleglob, regex)
     else:
         for path in glob.iglob(pathname):
             if re.search(regex, path):
@@ -601,7 +623,8 @@ def guess_overwrite_paths():
     elif 'nt' == os.name:
         localtmp = os.path.expandvars('$TMP')
         if not os.path.exists(localtmp):
-            logger.warning(_("The environment variable TMP refers to a directory that does not exist: %s"), localtmp)
+            logger.warning(
+                _("The environment variable TMP refers to a directory that does not exist: %s"), localtmp)
             localtmp = None
         from bleachbit.Windows import get_fixed_drives
         for drive in get_fixed_drives():
@@ -645,20 +668,10 @@ def is_dir_empty(dirname):
 
     It assumes the path exists and is a directory.
     """
-    if hasattr(os, 'scandir') and sys.version_info < (3,6,0):
-        # Python 3.5 added os.scandir() without context manager.
-        for entry in os.scandir(dirname):
+    with os.scandir(dirname) as it:
+        for _entry in it:
             return False
-        return True
-    elif hasattr(os, 'scandir'):
-        # Python 3.6 added the context manager.
-        with os.scandir(dirname) as it:
-            for entry in it:
-                return False
-        return True
-
-    # This method is slower, but it works with Python 3.4.
-    return len(os.listdir(dirname)) == 0
+    return True
 
 
 def listdir(directory):
@@ -668,8 +681,7 @@ def listdir(directory):
 
     if type(directory) is tuple:
         for dirname in directory:
-            for pathname in listdir(dirname):
-                yield pathname
+            yield from listdir(dirname)
         return
     dirname = os.path.expanduser(directory)
     if not os.path.lexists(dirname):
@@ -704,7 +716,7 @@ def sync():
         rc = ctypes.cdll.LoadLibrary('libc.so.6').sync()
         if 0 != rc:
             logger.error('sync() returned code %d', rc)
-    if 'nt' == os.name:
+    elif 'nt' == os.name:
         import ctypes
         ctypes.cdll.LoadLibrary('msvcrt.dll')._flushall()
 
@@ -819,7 +831,6 @@ def wipe_contents(path, truncate=True):
     if 'nt' == os.name and IsUserAnAdmin():
         from bleachbit.WindowsWipe import file_wipe, UnsupportedFileSystemError
         import warnings
-        from bleachbit import _
         try:
             file_wipe(path)
         except pywinerror as e:
@@ -841,7 +852,7 @@ def wipe_contents(path, truncate=True):
                     pass
             # translate exception to mark file to deletion in Command.py
             raise WindowsError(e.winerror, e.strerror)
-        except UnsupportedFileSystemError as e:
+        except UnsupportedFileSystemError:
             warnings.warn(
                 _('There was at least one file on a file system that does not support advanced overwriting.'), UserWarning)
             f = wipe_write()
@@ -865,7 +876,7 @@ def wipe_name(pathname1):
     while True:
         try:
             pathname2 = os.path.join(head, __random_string(maxlen))
-            os.rename(pathname1,  pathname2)
+            os.rename(pathname1, pathname2)
             break
         except OSError:
             if maxlen > 10:
@@ -939,115 +950,142 @@ def wipe_path(pathname, idle=False):
         remaining_seconds = int(remaining_bytes / (rate + 0.0001))
         return 1, done_percent, remaining_seconds
 
-    logger.debug(_("Wiping path: %s") % pathname)
+    # Get the file system type from the given path
+    fstype = get_filesystem_type(pathname)[0]
+    logger.debug(_(f"Wiping path {pathname} with file system type {fstype}"))
+    if not os.path.isdir(pathname):
+        logger.error(
+            _("Path to wipe must be an existing directory: %s"), pathname)
+        return
+
     files = []
     total_bytes = 0
     start_free_bytes = free_space(pathname)
     start_time = time.time()
-    # Because FAT32 has a maximum file size of 4,294,967,295 bytes,
-    # this loop is sometimes necessary to create multiple files.
-    while True:
-        try:
-            logger.debug(
-                _('Creating new, temporary file for wiping free space.'))
-            f = temporaryfile()
-        except OSError as e:
-            # Linux gives errno 24
-            # Windows gives errno 28 No space left on device
-            if e.errno in (errno.EMFILE, errno.ENOSPC):
-                break
-            else:
-                raise
+    done_wiping = False
+    try:
 
-        # Get the file system type from the given path
-        fstype = get_filesystem_type(pathname)
-        fstype = fstype[0]
-        logging.debug('File System:' + fstype)
-        # print(f.name) # Added by Marvin for debugging #issue 1051
-        last_idle = time.time()
-        # Write large blocks to quickly fill the disk.
-        blanks = b'\0' * 65536
-        writtensize = 0
-        
+        # Because FAT32 has a maximum file size of 4,294,967,295 bytes,
+        # this loop is sometimes necessary to create multiple files.
         while True:
             try:
-                if fstype != 'vfat':
-                    f.write(blanks)
-                # In the ubuntu system, the size of file should be less then 4GB. If not, there should be EFBIG error.
-                # So the maximum file size should be less than or equal to "4GB - 65536byte".
-                elif writtensize < 4 * 1024 * 1024 * 1024 - 65536:
-                    writtensize += f.write(blanks)
-                else:
-                    break
-            
-            except IOError as e:
-                if e.errno == errno.ENOSPC:
-                    if len(blanks) > 1:
-                        # Try writing smaller blocks
-                        blanks = blanks[0:len(blanks) // 2]
-                    else:
-                        break
-                elif e.errno == errno.EFBIG:
+                logger.debug(
+                    _('Creating new, temporary file for wiping free space.'))
+                f = temporaryfile()
+            except OSError as e:
+                # Linux gives errno 24
+                # Windows gives errno 28 No space left on device
+                if e.errno in (errno.EMFILE, errno.ENOSPC):
                     break
                 else:
                     raise
-            if idle and (time.time() - last_idle) > 2:
-                # Keep the GUI responding, and allow the user to abort.
-                # Also display the ETA.
-                yield estimate_completion()
-                last_idle = time.time()
-        # Write to OS buffer
-        try:
-            f.flush()
-        except IOError as e:
-            # IOError: [Errno 28] No space left on device
-            # seen on Microsoft Windows XP SP3 with ~30GB free space but
-            # not on another XP SP3 with 64MB free space
-            if not e.errno == errno.ENOSPC:
-                logger.error(
-                    _("Error #%d when flushing the file buffer." % e.errno))
 
-        os.fsync(f.fileno())  # write to disk
-        # Remember to delete
-        files.append(f)
-        # For statistics
-        total_bytes += f.tell()
-        # If no bytes were written, then quit.
-        # See https://github.com/bleachbit/bleachbit/issues/502
-        if start_free_bytes - total_bytes < 2: # Modified by Marvin to fix the issue #1051 [12/06/2020]
-            break
-    # sync to disk
-    sync()
-    # statistics
-    elapsed_sec = time.time() - start_time
-    rate_mbs = (total_bytes / (1000 * 1000)) / elapsed_sec
-    logger.info(_('Wrote {files:,} files and {bytes:,} bytes in {seconds:,} seconds at {rate:.2f} MB/s').format(
-                files=len(files), bytes=total_bytes, seconds=int(elapsed_sec), rate=rate_mbs))
-    # how much free space is left (should be near zero)
-    if 'posix' == os.name:
-        stats = os.statvfs(pathname)
-        logger.info(_("{bytes:,} bytes and {inodes:,} inodes available to non-super-user").format(
-                    bytes=stats.f_bsize * stats.f_bavail, inodes=stats.f_favail))
-        logger.info(_("{bytes:,} bytes and {inodes:,} inodes available to super-user").format(
-                    bytes=stats.f_bsize * stats.f_bfree, inodes=stats.f_ffree))
-    # truncate and close files
-    for f in files:
-        truncate_f(f)
+            # Remember to delete
+            files.append(f)
+            last_idle = time.time()
+            # Write large blocks to quickly fill the disk.
+            blanks = b'\0' * 65536
+            writtensize = 0
 
-        while True:
+            while True:
+
+                try:
+                    if fstype != 'vfat':
+                        f.write(blanks)
+                    # On Ubuntu, the size of file should be less then 4GB. If not, there should be EFBIG error,
+                    # so the maximum file size should be less than or equal to "4GB - 65536byte".
+                    elif writtensize < 4 * 1024 * 1024 * 1024 - 65536:
+                        writtensize += f.write(blanks)
+                    else:
+                        break
+
+                except IOError as e:
+                    if e.errno == errno.ENOSPC:
+                        if len(blanks) > 1:
+                            # Try writing smaller blocks
+                            blanks = blanks[0:len(blanks) // 2]
+                        else:
+                            break
+                    elif e.errno == errno.EFBIG:
+                        break
+                    else:
+                        raise
+                if idle and (time.time() - last_idle) > 2:
+                    # Keep the GUI responding, and allow the user to abort.
+                    # Also display the ETA.
+                    yield estimate_completion()
+                    last_idle = time.time()
+            # Write to OS buffer
             try:
-                # Nikita: I noticed a bug that prevented file handles from
-                # being closed on FAT32. It sometimes takes two .close() calls
-                # to do actually close (and therefore delete) a temporary file
-                f.close()
-                break
+                f.flush()
             except IOError as e:
-                if e.errno == 0:
-                    logger.debug(
-                        _("Handled unknown error #0 while truncating file."))
+                # IOError: [Errno 28] No space left on device
+                # seen on Microsoft Windows XP SP3 with ~30GB free space but
+                # not on another XP SP3 with 64MB free space
+                if not e.errno == errno.ENOSPC:
+                    logger.error(
+                        _("Error #%d when flushing the file buffer." % e.errno))
+
+            os.fsync(f.fileno())  # write to disk
+            # For statistics
+            total_bytes += f.tell()
+            # sync to disk
+            sync()
+            # statistics
+            elapsed_sec = time.time() - start_time
+            rate_mbs = (total_bytes / (1000 * 1000)) / elapsed_sec
+            logger.debug(_('Wrote {files:,} files and {bytes:,} bytes in {seconds:,} seconds at {rate:.2f} MB/s').format(
+                files=len(files), bytes=total_bytes, seconds=int(elapsed_sec), rate=rate_mbs))
+            # how much free space is left (should be near zero)
+            if 'posix' == os.name:
+                stats = os.statvfs(pathname)
+                logger.debug(_("{bytes:,} bytes and {inodes:,} inodes available to non-super-user").format(
+                    bytes=stats.f_bsize * stats.f_bavail, inodes=stats.f_favail))
+                logger.debug(_("{bytes:,} bytes and {inodes:,} inodes available to super-user").format(
+                    bytes=stats.f_bsize * stats.f_bfree, inodes=stats.f_ffree))
+            # If no bytes were written to this file, then do not try to create another file.
+            # Linux allows writing several 4K files when free_space() = 0,
+            # so do not check free_space() < 1.
+            # See
+            #  * https://github.com/bleachbit/bleachbit/issues/502
+            #    Replace `f.tell() < 2` with `len(blanks) < 2`
+            #  * https://github.com/bleachbit/bleachbit/issues/1051
+            #    Replace `len(blanks) < 2` with `estimated_free_space < 2`
+            estimated_free_space = start_free_bytes - total_bytes
+            if estimated_free_space < 2:
+                logger.debug(
+                    f'Estimated free space {estimated_free_space} is less than 2 bytes, breaking')
+                break
+        done_wiping = True
+    finally:
+        # Ensure files are closed and deleted even if an exception occurs or generator is not fully consumed.
+        # Truncate and close files.
+        for f in files:
+            if done_wiping:
+                try:
+                    truncate_f(f)
+                except Exception as e:
+                    logger.error(
+                        f'After wiping, truncating file {f.name} failed: {e}')
+
+            while True:
+                try:
+                    # Nikita: I noticed a bug that prevented file handles from
+                    # being closed on FAT32. It sometimes takes two .close() calls
+                    # to do actually close (and therefore delete) a temporary file
+                    f.close()
+                    break
+                except IOError as e:
+                    if e.errno == 0:
+                        logger.debug(
+                            _("Handled unknown error #0 while truncating file."))
                     time.sleep(0.1)
-        # explicitly delete
-        delete(f.name, ignore_missing=True)
+            # explicitly delete
+            try:
+                delete(f.name, ignore_missing=True)
+            except Exception as e:
+                logger.error(
+                    f'After wiping, error deleting file {f.name}: {e}')
 
 
 def vacuum_sqlite3(path):
