@@ -1,8 +1,7 @@
-#!/usr/bin/python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,15 +21,7 @@
 GTK graphical user interface
 """
 
-from bleachbit import GuiBasic
-from bleachbit import Cleaner, FileUtilities
-from bleachbit import _, APP_NAME, appicon_path, portable_mode, windows10_theme_path
-from bleachbit.Options import options
-from bleachbit.GuiPreferences import PreferencesDialog
-from bleachbit.Cleaner import backends, register_cleaners
-import bleachbit
-from gi.repository import Gtk, Gdk, GObject, GLib, Gio
-
+# standard library
 import glob
 import logging
 import os
@@ -38,16 +29,97 @@ import sys
 import threading
 import time
 
+
+# third party import
 import gi
-gi.require_version('Gtk', '3.0')
+gi.require_version('Gtk', '3.0')  # Keep this above `import Gtk`.
+from gi.repository import Gtk, Gdk, GObject, GLib, Gio
 
+APP_INDICATOR_FOUND = True
 
+if sys.platform == 'linux':
+    try:
+        # Ubuntu: sudo apt install gir1.2-ayatanaappindicator3-0.1
+        gi.require_version('AyatanaAppIndicator3', '0.1')  # throws ValueError
+        from gi.repository import AyatanaAppIndicator3 as AppIndicator
+    except (ValueError, ImportError):
+        try:
+            from gi.repository import AppIndicator3 as AppIndicator
+        except ImportError:
+            try:
+                from gi.repository import AppIndicator
+            except ImportError:
+                APP_INDICATOR_FOUND = False
+
+# local
+import bleachbit
+from bleachbit import APP_NAME, appicon_path, portable_mode, windows10_theme_path
+from bleachbit import Cleaner, FileUtilities, GuiBasic
+from bleachbit.Cleaner import backends, register_cleaners
+from bleachbit.GuiPreferences import PreferencesDialog
+from bleachbit.Language import get_text as _
+from bleachbit.Log import set_root_log_level
+from bleachbit.Options import options
 if os.name == 'nt':
     from bleachbit import Windows
 
+# Now that the configuration is loaded, honor the debug preference there.
+set_root_log_level(options.get('debug'))
 logger = logging.getLogger(__name__)
 
 
+class WindowInfo:
+    def __init__(self, x, y, width, height, monitor_model):
+        super().__init__()
+        self.x = x
+        self.y = y
+        self.width = width
+        self.height = height
+        self.monitor_model = monitor_model
+
+    def __str__(self):
+        return f"WindowInfo(x={self.x}, y={self.y}, width={self.width}, height={self.height}, monitor_model={self.monitor_model})"
+
+
+def get_window_info(window):
+    """Get the geometry and monitor of a window.
+
+    window: Gtk.Window
+
+    https://docs.gtk.org/gdk3/method.Screen.get_monitor_at_window.html
+    Deprecated since: 3.22
+    Use gdk_display_get_monitor_at_window() instead.
+
+    https://docs.gtk.org/gdk3/method.Display.get_monitor_at_window.html
+    Available since: 3.22
+
+    https://docs.gtk.org/gdk3/method.Screen.get_monitor_geometry.html
+    Deprecated since: 3.22
+    Use gdk_monitor_get_geometry() instead.
+
+    https://docs.gtk.org/gdk3/method.Monitor.get_geometry.html
+    Available since: 3.22
+
+    Returns a Rectangle-like object with with extra `monitor_model`
+    property with the monitor model string.
+    """
+    assert window is not None
+    assert isinstance(window, Gtk.Window)
+    gdk_window = window.get_window()
+    display = Gdk.Display.get_default()
+    assert display is not None
+    monitor = display.get_monitor_at_window(gdk_window)
+    assert monitor is not None
+    geo = monitor.get_geometry()
+    assert geo is not None
+    assert isinstance(geo, Gdk.Rectangle)
+    if display.get_n_monitors() > 0 and monitor.get_model():
+        monitor_model = monitor.get_model()
+    else:
+        monitor_model = "(unknown)"
+    return WindowInfo(geo.x, geo.y, geo.width, geo.height, monitor_model)
+
+
 def threaded(func):
     """Decoration to create a threaded function"""
     def wrapper(*args):
@@ -61,12 +133,20 @@ def notify_gi(msg):
 
     The Windows pygy-aio installer does not include notify, so this is just for Linux.
     """
-    gi.require_version('Notify', '0.7')
+    try:
+        gi.require_version('Notify', '0.7')
+    except ValueError as e:
+        logger.debug('gi.require_version("Notify", "0.7") failed: %s', e)
+        return
     from gi.repository import Notify
     if Notify.init(APP_NAME):
         notify = Notify.Notification.new('BleachBit', msg, 'bleachbit')
         notify.set_hint("desktop-entry", GLib.Variant('s', 'bleachbit'))
-        notify.show()
+        try:
+            notify.show()
+        except gi.repository.GLib.GError as e:
+            logger.debug('Notify.Notification.show() failed: %s', e)
+            return
         notify.set_timeout(10000)
 
 
@@ -117,16 +197,18 @@ class Bleachbit(Gtk.Application):
 
     def __init__(self, uac=True, shred_paths=None, auto_exit=False):
 
-        application_id_suffix = self._init_windows_misc(auto_exit, shred_paths, uac)
-        application_id = '{}{}'.format('org.gnome.Bleachbit', application_id_suffix)
+        application_id_suffix = self._init_windows_misc(
+            auto_exit, shred_paths, uac)
+        application_id = '{}{}'.format(
+            'org.gnome.Bleachbit', application_id_suffix)
         Gtk.Application.__init__(
             self, application_id=application_id, flags=Gio.ApplicationFlags.FLAGS_NONE)
-        GObject.threads_init()
+        GLib.set_prgname('org.bleachbit.BleachBit')
 
         if auto_exit:
             # This is used for automated testing of whether the GUI can start.
-            # It is called from assert_execute_console() in windows/setup_py2exe.py
-            self._auto_exit = True        
+            # It is called from assert_execute_console() in windows/setup.py
+            self._auto_exit = True
 
         if shred_paths:
             self._shred_paths = shred_paths
@@ -136,29 +218,21 @@ class Bleachbit(Gtk.Application):
             import atexit
             atexit.register(Windows.cleanup_nonce)
 
-            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
-            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # https://www.bleachbit.org/forum/074-fails-errors
-            try:
-                import sqlite3
-            except ImportError:
-                logger.exception(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."))
-
     def _init_windows_misc(self, auto_exit, shred_paths, uac):
         application_id_suffix = ''
         is_context_menu_executed = auto_exit and shred_paths
-        if os.name == 'nt':
-            if Windows.elevate_privileges(uac):
-                # privileges escalated in other process
-                sys.exit(0)
+        if not os.name == 'nt':
+            return ''
+        if Windows.elevate_privileges(uac):
+            # privileges escalated in other process
+            sys.exit(0)
 
-            if is_context_menu_executed:
-                # When we have a running application and executing the Windows
-                # context menu command we start a new process with new application_id.
-                # That is because the command line arguments of the context menu command
-                # are not passed to the already running instance.
-                application_id_suffix = 'ContextMenuShred'
+        if is_context_menu_executed:
+            # When we have a running application and executing the Windows
+            # context menu command we start a new process with new application_id.
+            # That is because the command line arguments of the context menu command
+            # are not passed to the already running instance.
+            application_id_suffix = 'ContextMenuShred'
         return application_id_suffix
 
     def build_app_menu(self):
@@ -167,10 +241,13 @@ class Bleachbit(Gtk.Application):
         On Linux with GTK 3.24, this code is necessary but not sufficient for
         the menu to work. The headerbar code is also needed.
 
-        On Windows with GTK 3.18, this cde is sufficient for the menu to work.
+        On Windows with GTK 3.18, this code is sufficient for the menu to work.
         """
-
+        from bleachbit.Language import setup_translation
+        setup_translation()
         builder = Gtk.Builder()
+        # set_translation_domain() seems to have no effect.
+        # builder.set_translation_domain('bleachbit')
         builder.add_from_file(bleachbit.app_menu_filename)
         menu = builder.get_object('app-menu')
         self.set_app_menu(menu)
@@ -228,17 +305,24 @@ class Bleachbit(Gtk.Application):
         clipboard.request_targets(self.cb_clipboard_uri_received)
 
     def cb_clipboard_uri_received(self, clipboard, targets, data):
-        """Callback for when URIs are received from clipboard"""
+        """Callback for when URIs are received from clipboard
+
+        With GTK 3.18.9 on Windows, there was no text/uri-list in targets,
+        but there is with GTK 3.24.34. However, Windows does not have
+        get_uris().
+        """
         shred_paths = None
-        if Gdk.atom_intern_static_string('text/uri-list') in targets:
+        if 'nt' == os.name and Gdk.atom_intern_static_string('FileNameW') in targets:
+            # Windows
+            # Use non-GTK+ functions because because GTK+ 2 does not work.
+            shred_paths = Windows.get_clipboard_paths()
+        elif Gdk.atom_intern_static_string('text/uri-list') in targets:
             # Linux
             shred_uris = clipboard.wait_for_contents(
                 Gdk.atom_intern_static_string('text/uri-list')).get_uris()
             shred_paths = FileUtilities.uris_to_paths(shred_uris)
-        elif Gdk.atom_intern_static_string('FileNameW') in targets:
-            # Windows
-            # Use non-GTK+ functions because because GTK+ 2 does not work.
-            shred_paths = Windows.get_clipboard_paths()
+        else:
+            logger.warning(_('No paths found in clipboard.'))
         if shred_paths:
             GUI.shred_paths(self._window, shred_paths)
         else:
@@ -301,8 +385,8 @@ class Bleachbit(Gtk.Application):
         GUI.update_log_level(self._window)
 
     def get_about_dialog(self):
-        dialog = Gtk.AboutDialog(comments='Program to clean unnecessary files',
-                                 copyright='Copyright (C) 2008-2021 Andrew Ziem',
+        dialog = Gtk.AboutDialog(comments=_("Program to clean unnecessary files"),
+                                 copyright=bleachbit.APP_COPYRIGHT,
                                  program_name=APP_NAME,
                                  version=bleachbit.APP_VERSION,
                                  website=bleachbit.APP_URL,
@@ -313,7 +397,6 @@ class Bleachbit(Gtk.Application):
         except (IOError, TypeError):
             dialog.set_license(
                 _("GNU General Public License version 3 or later.\nSee https://www.gnu.org/licenses/gpl-3.0.txt"))
-        # dialog.set_name(APP_NAME)
         # TRANSLATORS: Maintain the names of translators here.
         # Launchpad does this automatically for translations
         # typed in Launchpad. This is a special string shown
@@ -375,7 +458,8 @@ class Bleachbit(Gtk.Application):
                 application=self, title=APP_NAME, auto_exit=self._auto_exit)
         self._window.present()
         if self._shred_paths:
-            GLib.idle_add(GUI.shred_paths, self._window, self._shred_paths, priority=GObject.PRIORITY_LOW)
+            GLib.idle_add(GUI.shred_paths, self._window,
+                          self._shred_paths, priority=GObject.PRIORITY_LOW)
             # When we shred paths and auto exit with the Windows Explorer context menu command we close the
             # application in GUI.shred_paths, because if it is closed from here there are problems.
             # Most probably this is something related with how GTK handles idle quit calls.
@@ -435,9 +519,16 @@ class TreeInfoModel:
                                                               self.on_row_changed)
 
     def sort_func(self, model, iter1, iter2, _user_data):
-        """Sort the tree by the display name"""
-        value1 = model[iter1][0].lower()
-        value2 = model[iter2][0].lower()
+        """Sort the tree by the id
+
+        Index 0 is the display name
+        Index 2 is the ID (e.g., cookies, vacuum).
+
+        Sorting by ID is functionally important, so that vacuuming is done
+        last, even for other languages. See https://github.com/bleachbit/bleachbit/issues/441
+        """
+        value1 = model[iter1][2].lower()
+        value2 = model[iter2][2].lower()
         if value1 == value2:
             return 0
         if value1 > value2:
@@ -513,7 +604,9 @@ class TreeDisplayModel:
             if warning:
                 resp = GuiBasic.message_dialog(parent_window,
                                                msg,
-                                               Gtk.MessageType.WARNING, Gtk.ButtonsType.OK_CANCEL)
+                                               Gtk.MessageType.WARNING,
+                                               Gtk.ButtonsType.OK_CANCEL,
+                                               _('Confirm'))
                 if Gtk.ResponseType.OK != resp:
                     # user cancelled, so don't toggle option
                     return
@@ -559,7 +652,8 @@ class GUI(Gtk.ApplicationWindow):
 
         self._auto_exit = auto_exit
 
-        self.set_wmclass(APP_NAME, APP_NAME)
+        self.set_property('name', APP_NAME)
+        self.set_property('role', APP_NAME)
         self.populate_window()
 
         # Redirect logging to the GUI.
@@ -587,13 +681,49 @@ class GUI(Gtk.ApplicationWindow):
 
         GLib.idle_add(self.cb_refresh_operations)
 
+        # Close the application when user presses CTRL+Q or CTRL+W.
+        accel = Gtk.AccelGroup()
+        self.add_accel_group(accel)
+        key, mod = Gtk.accelerator_parse("<Control>Q")
+        accel.connect(key, mod, Gtk.AccelFlags.VISIBLE, self.on_quit)
+        key, mod = Gtk.accelerator_parse("<Control>W")
+        accel.connect(key, mod, Gtk.AccelFlags.VISIBLE, self.on_quit)
+
+        self._set_appindicator()
+
+    def _set_appindicator(self):
+        """Setup the app indicator"""
+        if not (sys.platform == 'linux' and APP_INDICATOR_FOUND):
+            return
+        APPINDICATOR_ID = 'BLEACHBIT'
+        icon_dir = os.path.dirname(appicon_path)
+        menu_icon_path = os.path.join(icon_dir, 'bleachbit-indicator.svg')
+        if not os.path.exists(menu_icon_path):
+            if os.path.exists(appicon_path):
+                menu_icon_path = appicon_path
+            else:
+                menu_icon_path = 'user-trash'
+        indicator_category = AppIndicator.IndicatorCategory.SYSTEM_SERVICES
+        self.indicator = AppIndicator.Indicator.new(
+            APPINDICATOR_ID, menu_icon_path, indicator_category)
+        self.indicator.set_status(AppIndicator.IndicatorStatus.ACTIVE)
+        self.indicator.set_menu(self.build_appindicator_menu())
+
+    def on_quit(self, *args):
+        """Quit the application, used with CTRL+Q or CTRL+W"""
+        if Gtk.main_level() > 0:
+            Gtk.main_quit()
+        else:
+            self.destroy()
+
     def _show_splash_screen(self):
+        """Show the splash screen on Windows because startup may be slow"""
         if os.name != 'nt':
             return
 
         font_conf_file = Windows.get_font_conf_file()
         if not os.path.exists(font_conf_file):
-            logger.error('No fonts.conf file')
+            logger.error('No fonts.conf file {}'.format(font_conf_file))
             return
 
         has_cache = Windows.has_fontconfig_cache(font_conf_file)
@@ -605,6 +735,23 @@ class GUI(Gtk.ApplicationWindow):
             return GuiBasic.delete_confirmation_dialog(self, mention_preview, shred_settings=shred_settings)
         return True
 
+    def destroy(self):
+        """Prevent textbuffer usage during UI destruction"""
+        self.textbuffer = None
+        super(GUI, self).destroy()
+
+    def build_appindicator_menu(self):
+        """Build the app indicator menu"""
+        menu = Gtk.Menu()
+        item_clean = Gtk.MenuItem('Clean')
+        item_clean.connect('activate', self.run_operations)
+        item_quit = Gtk.MenuItem('Quit BleachBit')
+        item_quit.connect('activate', self.on_quit)
+        menu.append(item_clean)
+        menu.append(item_quit)
+        menu.show_all()
+        return menu
+
     def get_preferences_dialog(self):
         return PreferencesDialog(
             self,
@@ -633,13 +780,16 @@ class GUI(Gtk.ApplicationWindow):
 
         if self._auto_exit:
             GLib.idle_add(self.close,
-                              priority=GObject.PRIORITY_LOW)
+                          priority=GObject.PRIORITY_LOW)
 
         # user aborted
         return False
 
     def append_text(self, text, tag=None, __iter=None, scroll=True):
         """Add some text to the main log"""
+        if self.textbuffer is None:
+            # textbuffer was destroyed.
+            return
         if not __iter:
             __iter = self.textbuffer.get_end_iter()
         if tag:
@@ -650,7 +800,7 @@ class GUI(Gtk.ApplicationWindow):
         # through the idle loop, it may only scroll most of the way
         # as seen on Ubuntu 9.04 with Italian and Spanish.
         if scroll:
-            GLib.idle_add(lambda:
+            GLib.idle_add(lambda: self.textbuffer is not None and
                           self.textview.scroll_mark_onscreen(
                               self.textbuffer.get_insert()))
 
@@ -747,7 +897,9 @@ class GUI(Gtk.ApplicationWindow):
         if not operations:  # empty
             GuiBasic.message_dialog(self,
                                     _("You must select an operation"),
-                                    Gtk.MessageType.WARNING, Gtk.ButtonsType.OK)
+                                    Gtk.MessageType.ERROR,
+                                    Gtk.ButtonsType.OK,
+                                    _('Error'))
             return
         try:
             self.set_sensitive(False)
@@ -788,19 +940,24 @@ class GUI(Gtk.ApplicationWindow):
         scrolled_window = Gtk.ScrolledWindow()
         scrolled_window.set_policy(
             Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        scrolled_window.set_overlay_scrolling(False)
         self.tree_store = TreeInfoModel()
         display = TreeDisplayModel()
         mdl = self.tree_store.get_model()
         self.view = display.make_view(
             mdl, self, self.context_menu_event)
         self.view.get_selection().connect("changed", self.on_selection_changed)
-        scrollbar_width = scrolled_window.get_vscrollbar().get_preferred_width()[1]
-        self.view.set_margin_right(scrollbar_width) # avoid conflict with scrollbar
+        scrollbar_width = scrolled_window.get_vscrollbar().get_preferred_width()[
+            1]
+        # avoid conflict with scrollbar
+        self.view.set_margin_end(scrollbar_width)
         scrolled_window.add(self.view)
         return scrolled_window
 
     def cb_refresh_operations(self):
-        """Callback to refresh the list of cleaners"""
+        """Callback to refresh the list of cleaners and header bar labels"""
+        # In case language changed, update the header bar labels.
+        self.update_headerbar_labels()
         # Is this the first time in this session?
         if not hasattr(self, 'recognized_cleanerml') and not self._auto_exit:
             from bleachbit import RecognizeCleanerML
@@ -843,20 +1000,10 @@ class GUI(Gtk.ApplicationWindow):
                     _('Access the application menu by clicking the logo on the title bar.'))
             options.set('first_start', False)
 
-        if os.name == 'nt':
-            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
-            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # http://bleachbit.sourceforge.net/forum/074-fails-errors
-            try:
-                import sqlite3
-            except ImportError as e:
-                self.append_text(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."), 'error')
-
         # Show notice about admin privileges.
         if os.name == 'posix' and os.path.expanduser('~') == '/root':
             self.append_text(
-                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.')+'\n')
+                _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.') + '\n')
         if os.name == 'nt' and options.get('shred'):
             from win32com.shell.shell import IsUserAnAdmin
             if not IsUserAnAdmin():
@@ -864,6 +1011,10 @@ class GUI(Gtk.ApplicationWindow):
                     _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
                 self.append_text('\n')
 
+        if 'windowsapps' in sys.executable.lower():
+            self.append_text(
+                _('There is no official version of BleachBit on the Microsoft Store. Get the genuine version at https://www.bleachbit.org where it is always free of charge.') + '\n', 'error')
+
         # remove from idle loop (see GObject.idle_add)
         return False
 
@@ -960,7 +1111,7 @@ class GUI(Gtk.ApplicationWindow):
         treepath = Gtk.TreePath(0)
         try:
             __iter = model.get_iter(treepath)
-        except ValueError as e:
+        except ValueError:
             logger.warning(
                 'ValueError in get_iter() when updating file size for tree path=%s' % treepath)
             return
@@ -984,9 +1135,81 @@ class GUI(Gtk.ApplicationWindow):
             text = ""
         self.status_bar.push(context_id, text)
 
+    def update_headerbar_labels(self):
+        """Update the labels and tooltips in the headerbar buttons"""
+        # Preview button
+        self.preview_button.set_tooltip_text(
+            _("Preview files in the selected operations (without deleting any files)"))
+        # TRANSLATORS: This is the preview button on the main window.  It
+        # previews changes.
+        self.preview_button.set_label(_('Preview'))
+
+        # Clean button
+        # TRANSLATORS: This is the clean button on the main window.
+        # It makes permanent changes: usually deleting files, sometimes
+        # altering them.
+        self.run_button.set_label(_('Clean'))
+        self.run_button.set_tooltip_text(
+            _("Clean files in the selected operations"))
+
+        # Stop button
+        self.stop_button.set_label(_('Abort'))
+        self.stop_button.set_tooltip_text(
+            _('Abort the preview or cleaning process'))
+
+    def on_update_button_clicked(self, widget):
+        """Callback when the update button on the headerbar is clicked"""
+        if not (hasattr(self, '_available_updates') and self._available_updates):
+            return
+
+        self.update_button.get_style_context().remove_class('update-available')
+        updates = self._available_updates
+        if len(updates) == 1:
+            ver, url = updates[0]
+            GuiBasic.open_url(url, self, False)
+            return
+        # If multiple updates are available, find out which one the user wants.
+        from bleachbit import Update
+        Update.update_dialog(self, updates)
+
     def create_headerbar(self):
         """Create the headerbar"""
         hbar = Gtk.HeaderBar()
+
+        # The update button is on the right side of the headerbar.
+        # It is hidden until an update is available.
+        self.update_button = Gtk.Button()
+        self.update_button.set_visible(False)
+        self.update_button.connect('clicked', self.on_update_button_clicked)
+        # TRANSLATORS: Button in headerbar to update the application
+        self.update_button.set_label(_('Update'))
+        self.update_button.set_tooltip_text(
+            _('Update BleachBit to the latest version'))
+
+        # Add CSS to animate the update button.
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(b"""
+            @keyframes update-pulse {
+                0% { opacity: 1; }
+                50% { opacity: 0.7; }
+                100% { opacity: 1; }
+            }
+            .update-available {
+                background: @theme_selected_bg_color;
+                color: @theme_selected_fg_color;
+                animation: update-pulse 2s ease-in-out;
+                animation-delay: 2s;
+                animation-iteration-count: 1;
+            }
+        """)
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            css_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+        self.update_button.get_style_context().add_class('update-available')
+        hbar.pack_end(self.update_button)
+        self.update_button.set_no_show_all(True)
+        self.update_button.hide()
         hbar.props.show_close_button = True
         hbar.props.title = APP_NAME
 
@@ -1004,23 +1227,12 @@ class GUI(Gtk.ApplicationWindow):
         self.preview_button.set_always_show_image(True)
         self.preview_button.connect(
             'clicked', lambda *dummy: self.preview_or_run_operations(False))
-        self.preview_button.set_tooltip_text(
-            _("Preview files in the selected operations (without deleting any files)"))
-        # TRANSLATORS: This is the preview button on the main window.  It
-        # previews changes.
-        self.preview_button.set_label(_('Preview'))
         box.add(self.preview_button)
 
         # create the delete button
         self.run_button = Gtk.Button.new_from_icon_name(
             'edit-clear-all', icon_size)
         self.run_button.set_always_show_image(True)
-        # TRANSLATORS: This is the clean button on the main window.
-        # It makes permanent changes: usually deleting files, sometimes
-        # altering them.
-        self.run_button.set_label(_('Clean'))
-        self.run_button.set_tooltip_text(
-            _("Clean files in the selected operations"))
         self.run_button.connect("clicked", self.run_operations)
         box.add(self.run_button)
 
@@ -1028,9 +1240,6 @@ class GUI(Gtk.ApplicationWindow):
         self.stop_button = Gtk.Button.new_from_icon_name(
             'process-stop', icon_size)
         self.stop_button.set_always_show_image(True)
-        self.stop_button.set_label(_('Abort'))
-        self.stop_button.set_tooltip_text(
-            _('Abort the preview or cleaning process'))
         self.stop_button.set_sensitive(False)
         self.stop_button.connect('clicked', self.cb_stop_operations)
         box.add(self.stop_button)
@@ -1051,6 +1260,8 @@ class GUI(Gtk.ApplicationWindow):
         menu_button.add(image)
         hbar.pack_end(menu_button)
 
+        # Update all labels and tooltips
+        self.update_headerbar_labels()
         return hbar
 
     def on_configure_event(self, widget, event):
@@ -1062,12 +1273,10 @@ class GUI(Gtk.ApplicationWindow):
         if 'nt' == os.name:
             window = self.get_window()
             if window.get_state() & Gdk.WindowState.MAXIMIZED != 0:
-                screen = self.get_screen()
-                monitor_num = screen.get_monitor_at_window(window)
-                g = screen.get_monitor_geometry(monitor_num)
+                g = get_window_info(self)
                 if x < g.x or x >= g.x + g.width or y < g.y or y >= g.y + g.height:
-                    logger.debug("Maximized window {}+{}: monitor ({}) geometry = {}+{}".format(
-                        (x, y), (width, height), monitor_num, (g.x, g.y), (g.width, g.height)))
+                    logger.debug("Maximized window {}+{}: {}".format(
+                        (x, y), (width, height), str(g)))
                     self.move(g.x, g.y)
                     return True
 
@@ -1079,11 +1288,30 @@ class GUI(Gtk.ApplicationWindow):
         return False
 
     def on_window_state_event(self, widget, event):
-        # save window state
-        fullscreen = event.new_window_state & Gdk.WindowState.FULLSCREEN != 0
+        """Save window state
+
+        GTK version 3.24.34 on Windows 11 behaves strangely:
+        * It reports maximized only when application starts.
+        * Later, it reports window is fullscreen when neither
+          full screen nor maximized.
+
+        Because of this issue, we check the tiling state.
+        """
+        tiling_states = (Gdk.WindowState.TILED |
+                         Gdk.WindowState.TOP_TILED |
+                         Gdk.WindowState.RIGHT_TILED |
+                         Gdk.WindowState.BOTTOM_TILED |
+                         Gdk.WindowState.LEFT_TILED)
+
+        is_tiled = event.new_window_state & tiling_states != 0
+        fullscreen = (event.new_window_state &
+                      Gdk.WindowState.FULLSCREEN != 0) and not is_tiled
         options.set("window_fullscreen", fullscreen, commit=False)
         maximized = event.new_window_state & Gdk.WindowState.MAXIMIZED != 0
         options.set("window_maximized", maximized, commit=False)
+        if 'nt' == os.name:
+            logger.debug(
+                f'window state = {event.new_window_state}, full screen = {fullscreen}, maximized = {maximized}')
         return False
 
     def on_delete_event(self, widget, event):
@@ -1091,9 +1319,13 @@ class GUI(Gtk.ApplicationWindow):
         options.commit()
         return False
 
-    def on_show(self, widget):
+    def on_show(self, _widget):
+        """Handle the show event.
 
-        if Windows.splash_thread.is_alive():
+        The event is triggered when the window is first shown.
+        It is not emitted when the window is moved or unminimized.
+        """
+        if 'nt' == os.name and Windows.splash_thread.is_alive():
             Windows.splash_thread.join(0)
 
         # restore window position, size and state
@@ -1106,16 +1338,14 @@ class GUI(Gtk.ApplicationWindow):
             (r.width, r.height) = (options.get(
                 "window_width"), options.get("window_height"))
 
-            screen = self.get_screen()
-            monitor_num = screen.get_monitor_at_point(r.x, r.y)
-            g = screen.get_monitor_geometry(monitor_num)
+            g = get_window_info(self)
 
             # only restore position and size if window left corner
             # is within the closest monitor
             if r.x >= g.x and r.x < g.x + g.width and \
                r.y >= g.y and r.y < g.y + g.height:
-                logger.debug("closest monitor ({}) geometry = {}+{}, window geometry = {}+{}".format(
-                    monitor_num, (g.x, g.y), (g.width, g.height), (r.x, r.y), (r.width, r.height)))
+                logger.debug("closest monitor {}, prior window geometry = {}+{}".format(
+                    str(g), (r.x, r.y), (r.width, r.height)))
                 self.move(r.x, r.y)
                 self.resize(r.width, r.height)
         if options.get("window_fullscreen"):
@@ -1154,8 +1384,18 @@ class GUI(Gtk.ApplicationWindow):
     def populate_window(self):
         """Create the main application window"""
         screen = self.get_screen()
-        self.set_default_size(min(screen.width(), 800),
-                              min(screen.height(), 600))
+        display = screen.get_display()
+        monitor = display.get_primary_monitor()
+        if monitor is None:
+            # See https://github.com/bleachbit/bleachbit/issues/1793
+            if display.get_n_monitors() > 0:
+                monitor = display.get_monitor(0)
+        if monitor is None:
+            self.set_default_size(800, 600)
+        else:
+            geometry = monitor.get_geometry()
+            self.set_default_size(min(geometry.width, 800),
+                                  min(geometry.height, 600))
         self.set_position(Gtk.WindowPosition.CENTER)
         self.connect("configure-event", self.on_configure_event)
         self.connect("window-state-event", self.on_window_state_event)
@@ -1237,8 +1477,14 @@ class GUI(Gtk.ApplicationWindow):
                                            options.get('update_winapp2'),
                                            self.append_text,
                                            lambda: GLib.idle_add(self.cb_refresh_operations))
-            if updates:
-                GLib.idle_add(
-                    lambda: Update.update_dialog(self, updates))
+
         except Exception:
             logger.exception(_("Error when checking for updates: "))
+        else:
+            self._available_updates = updates
+
+            def update_button_state():
+                if updates:
+                    self.update_button.show()
+                    self.set_sensitive(True)
+            GLib.idle_add(update_button_state)
