@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2023 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -32,7 +32,7 @@ from xml.dom.minidom import parseString
 import bleachbit
 from bleachbit import Cleaner, Windows
 from bleachbit.Action import Delete, Winreg
-from bleachbit import _
+from bleachbit.Language import get_text as _
 
 logger = logging.getLogger(__name__)
 
@@ -115,7 +115,7 @@ def winapp_expand_vars(pathname):
 def detect_file(pathname):
     """Check whether a path exists for DetectFile#="""
     for expanded in winapp_expand_vars(pathname):
-        for _ in glob.iglob(expanded):
+        for _i in glob.iglob(expanded):
             return True
     return False
 
@@ -172,7 +172,7 @@ class Winapp:
                 logger.exception('parsing error in section %s', section)
             else:
                 section_done_count += 1
-                cb_progress(1.0*section_done_count/section_total_count)
+                cb_progress(1.0 * section_done_count / section_total_count)
 
     def add_section(self, cleaner_id, name):
         """Add a section (cleaners)"""
@@ -180,6 +180,7 @@ class Winapp:
         self.cleaners[cleaner_id] = Cleaner.Cleaner()
         self.cleaners[cleaner_id].id = cleaner_id
         self.cleaners[cleaner_id].name = name
+        assert name.strip() == name
         self.cleaners[cleaner_id].description = _('Imported from winapp2.ini')
         # The detect() function in this module effectively does what
         # auto_hide() does, so this avoids redundant, slow processing.
@@ -307,8 +308,9 @@ class Winapp:
             return
         # find the BleachBit internal cleaner ID
         lid = self.section_to_cleanerid(langsecref_num)
+        option_name = section.replace('*', '').strip()
         self.cleaners[lid].add_option(
-            section2option(section), section.replace('*', ''), '')
+            section2option(section), option_name, '')
         for option in self.parser.options(section):
             if (
                 option
@@ -347,7 +349,8 @@ class Winapp:
                     search = 'walk.all'
             else:
                 import fnmatch
-                regex = ' regex="^%s$" ' % xml_escape(fnmatch.translate(filename))
+                regex = ' regex="^%s$" ' % xml_escape(
+                    fnmatch.translate(filename))
         else:
             search = 'glob'
             path = os.path.join(dirname, filename)
@@ -399,7 +402,8 @@ class Winapp:
             for dirname in dirnames:
                 # If dirname is a drive letter it needs a special treatment on Windows:
                 # https://www.reddit.com/r/learnpython/comments/gawqne/why_cant_i_ospathjoin_on_a_drive_letterc/
-                dirname = '{}{}'.format(dirname, os.path.sep) if os.path.splitdrive(dirname)[0] == dirname else dirname
+                dirname = '{}{}'.format(dirname, os.path.sep) if os.path.splitdrive(dirname)[
+                    0] == dirname else dirname
                 for provider in self.__make_file_provider(dirname, filename, recurse, removeself, excludekeys):
                     self.cleaners[lid].add_action(
                         section2option(ini_section), provider)
@@ -437,7 +441,7 @@ def load_cleaners(cb_progress=lambda x: None):
     for pathname in list_winapp_files():
         try:
             inicleaner = Winapp(pathname, cb_progress)
-        except Exception as e:
+        except Exception:
             logger.exception(
                 "Error reading winapp2.ini cleaner '%s'", pathname)
         else:
