@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -18,7 +18,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-"""
+r"""
 Functionality specific to Microsoft Windows
 
 The Windows Registry terminology can be confusing. Take for example
@@ -38,14 +38,16 @@ These are the terms:
 """
 
 import bleachbit
-from bleachbit import _, Command, FileUtilities, General
+from bleachbit import Command, FileUtilities, General
+from bleachbit.Language import get_text as _
 
+import errno
 import glob
 import logging
 import os
 import sys
 import shutil
-from threading import Thread
+from threading import Thread, Event
 import xml.dom.minidom
 
 from decimal import Decimal
@@ -60,7 +62,7 @@ if 'win32' == sys.platform:
     import win32process
     import win32security
 
-    from ctypes import windll, c_ulong, c_buffer, byref, sizeof
+    from ctypes import windll, byref
     from win32com.shell import shell, shellcon
 
     psapi = windll.psapi
@@ -115,7 +117,7 @@ def browse_files(_, title):
 
 def browse_folder(_, title):
     """Ask the user to select a folder.  Return full path."""
-    flags = 0x0010 #SHBrowseForFolder path input
+    flags = 0x0010  # SHBrowseForFolder path input
     pidl = shell.SHBrowseForFolder(None, None, title, flags)[0]
     if pidl is None:
         # user cancelled
@@ -126,7 +128,7 @@ def browse_folder(_, title):
 
 def cleanup_nonce():
     """On exit, clean up GTK junk files"""
-    for fn in glob.glob(os.path.expandvars('%TEMP%\gdbus-nonce-file-*')):
+    for fn in glob.glob(os.path.expandvars(r'%TEMP%\gdbus-nonce-file-*')):
         logger.debug('cleaning GTK nonce file: %s', fn)
         FileUtilities.delete(fn)
 
@@ -149,7 +151,14 @@ def delete_locked_file(pathname):
         MOVEFILE_DELAY_UNTIL_REBOOT = 4
         if 0 == windll.kernel32.MoveFileExW(pathname, None, MOVEFILE_DELAY_UNTIL_REBOOT):
             from ctypes import WinError
-            raise WinError()
+            # WinError throws the right exception based on last error.
+            try:
+                raise WinError()
+            except PermissionError:
+                # OSError has special handling in Worker.py
+                # Use a special message for flagging files for later deletion
+                raise OSError(
+                    errno.EACCES, "Access denied in delete_locked_file()", pathname)
 
 
 def delete_registry_value(key, value_name, really_delete):
@@ -158,26 +167,22 @@ def delete_registry_value(key, value_name, really_delete):
     successful.  If really_delete is False (meaning preview),
     just check whether the value exists."""
     (hive, sub_key) = split_registry_key(key)
-    if really_delete:
-        try:
+    try:
+        if really_delete:
             hkey = winreg.OpenKey(hive, sub_key, 0, winreg.KEY_SET_VALUE)
             winreg.DeleteValue(hkey, value_name)
-        except WindowsError as e:
-            if e.winerror == 2:
-                # 2 = 'file not found' means value does not exist
-                return False
-            raise
         else:
-            return True
-    try:
-        hkey = winreg.OpenKey(hive, sub_key)
-        winreg.QueryValueEx(hkey, value_name)
+            hkey = winreg.OpenKey(hive, sub_key)
+            winreg.QueryValueEx(hkey, value_name)
+    except PermissionError:
+        raise OSError(
+            errno.EACCES, "Access denied in delete_registry_value()", key)
     except WindowsError as e:
-        if e.winerror == 2:
+        if e.winerror == errno.ENOENT:
+            # ENOENT = 'file not found' means value does not exist
             return False
         raise
-    else:
-        return True
+    return True
 
 
 def delete_registry_key(parent_key, really_delete):
@@ -205,7 +210,11 @@ def delete_registry_key(parent_key, really_delete):
     ]
     for child_key in child_keys:
         delete_registry_key(child_key, True)
-    winreg.DeleteKey(hive, parent_sub_key)
+    try:
+        winreg.DeleteKey(hive, parent_sub_key)
+    except PermissionError:
+        raise OSError(
+            errno.EACCES, "Access denied in delete_registry_key()", parent_key)
     return True
 
 
@@ -218,7 +227,11 @@ def delete_updates():
     dirs += [os.path.expandvars(r'%windir%\SoftwareDistribution.bak')]
     dirs += [os.path.expandvars(r'%windir%\ie7updates')]
     dirs += [os.path.expandvars(r'%windir%\ie8updates')]
-    dirs += [os.path.expandvars(r'%windir%\system32\catroot2')]
+    # see https://github.com/bleachbit/bleachbit/issues/1215 about catroot2
+    # dirs += [os.path.expandvars(r'%windir%\system32\catroot2')]
+    dirs += [os.path.expandvars(r'%systemdrive%\windows.old')]
+    dirs += [os.path.expandvars(r'%systemdrive%\$windows.~bt')]
+    dirs += [os.path.expandvars(r'%systemdrive%\$windows.~ws')]
     if not dirs:
         # if nothing to delete, then also do not restart service
         return
@@ -284,8 +297,10 @@ def elevate_privileges(uac):
         htoken = win32security.OpenProcessToken(
             win32api.GetCurrentProcess(), win32security.TOKEN_ADJUST_PRIVILEGES | win32security.TOKEN_QUERY)
         newPrivileges = [
-            (win32security.LookupPrivilegeValue(None, "SeBackupPrivilege"), win32security.SE_PRIVILEGE_ENABLED),
-            (win32security.LookupPrivilegeValue(None, "SeRestorePrivilege"), win32security.SE_PRIVILEGE_ENABLED),
+            (win32security.LookupPrivilegeValue(None, "SeBackupPrivilege"),
+             win32security.SE_PRIVILEGE_ENABLED),
+            (win32security.LookupPrivilegeValue(None, "SeRestorePrivilege"),
+             win32security.SE_PRIVILEGE_ENABLED),
         ]
         win32security.AdjustTokenPrivileges(htoken, 0, newPrivileges)
         win32file.CloseHandle(htoken)
@@ -309,7 +324,6 @@ def elevate_privileges(uac):
         parameters = '"%s" --gui --no-uac' % pyfile
         exe = sys.executable
 
-
     parameters = _add_command_line_parameters(parameters)
 
     logger.debug('elevate_privileges() exe=%s, parameters=%s', exe, parameters)
@@ -483,23 +497,79 @@ def is_junction(path):
     return bool(attr & FILE_ATTRIBUTE_REPARSE_POINT)
 
 
-def is_process_running(name):
+def is_process_running(exename, require_same_user):
     """Return boolean whether process (like firefox.exe) is running
 
-    Works on Windows Vista or later, but on Windows XP gives an ImportError
+    exename: name of the executable
+    require_same_user: if True, ignore processes run by other users
     """
 
     import psutil
-    name = name.lower()
+    exename = exename.lower()
+    current_username = psutil.Process().username().lower()
     for proc in psutil.process_iter():
         try:
-            if proc.name().lower() == name:
-                return True
+            proc_name = proc.name().lower()
         except psutil.NoSuchProcess:
-            pass
+            continue
+        if not proc_name == exename:
+            continue
+        if not require_same_user:
+            return True
+        try:
+            proc_username = proc.username().lower()
+        except psutil.AccessDenied:
+            continue
+        if proc_username == current_username:
+            return True
     return False
 
 
+def load_i18n_dll():
+    """Load internationalization library
+
+    It may be called either libintl-8.dll or intl-8.dll, and it comes
+    from gettext.
+
+    Returns None if the dll is not available.
+    """
+    import ctypes
+    lib_fns = ['libintl-8.dll', 'intl-8.dll']
+    dirs = set([bleachbit.bleachbit_exe_path, os.path.dirname(sys.executable)])
+    lib_path = None
+    for lib_fn in lib_fns:
+        if lib_path:
+            break
+        for dir in dirs:
+            lib_path = os.path.join(dir, lib_fn)
+            if os.path.exists(lib_path):
+                break
+            lib_path = None
+    if not lib_path:
+        logger.warning(
+            'internationalization library was not found, so translations will not work.')
+        return
+    try:
+        libintl = ctypes.cdll.LoadLibrary(lib_path)
+    except Exception as e:
+        logger.warning('error in LoadLibrary(%s): %s', lib_path, e)
+        return
+
+    # Configure DLL function prototypes
+    libintl.bindtextdomain.argtypes = [ctypes.c_char_p, ctypes.c_char_p]
+    libintl.bindtextdomain.restype = ctypes.c_char_p
+    libintl.bind_textdomain_codeset.argtypes = [
+        ctypes.c_char_p, ctypes.c_char_p]
+    libintl.textdomain.argtypes = [ctypes.c_char_p]
+
+    if hasattr(libintl, "libintl_wbindtextdomain"):
+        libintl.libintl_wbindtextdomain.argtypes = [
+            ctypes.c_char_p, ctypes.c_wchar_p]
+        libintl.libintl_wbindtextdomain.restype = ctypes.c_wchar_p
+
+    return libintl
+
+
 def move_to_recycle_bin(path):
     """Move 'path' into recycle bin"""
     shell.SHFileOperation(
@@ -519,9 +589,9 @@ def parse_windows_build(build=None):
 
 def path_on_network(path):
     """Check whether 'path' is on a network drive"""
-    if len(os.path.splitunc(path)[0]) > 0:
+    drive = os.path.splitdrive(path)[0]
+    if drive.startswith(r'\\'):
         return True
-    drive = os.path.splitdrive(path)[0] + '\\'
     return win32file.GetDriveType(drive) == win32file.DRIVE_REMOTE
 
 
@@ -539,7 +609,7 @@ def set_environ(varname, path):
     if not path:
         return
     if varname in os.environ:
-        #logger.debug('set_environ(%s, %s): skipping because environment variable is already defined', varname, path)
+        # logger.debug('set_environ(%s, %s): skipping because environment variable is already defined', varname, path)
         if 'nt' == os.name:
             os.environ[varname] = os.path.expandvars('%%%s%%' % varname)
         # Do not redefine the environment variable when it already exists
@@ -556,7 +626,9 @@ def set_environ(varname, path):
 
 
 def setup_environment():
-    """Define any extra environment variables for use in CleanerML and Winapp2.ini"""
+    """Define any extra environment variables"""
+
+    # These variables are for use in CleanerML and Winapp2.ini.
     csidl_to_environ('commonappdata', shellcon.CSIDL_COMMON_APPDATA)
     csidl_to_environ('documents', shellcon.CSIDL_PERSONAL)
     # Windows XP does not define localappdata, but Windows Vista and 7 do
@@ -578,6 +650,30 @@ def setup_environment():
     # cmd.exe .
     set_environ('cd', os.getcwd())
 
+    # XDG_DATA_DIRS environment variable needs to be set with both GTK icons and
+    # `glib-2.0\schemas\gschemas.compiled`.
+    # The latter is required by the make chaff dialog.
+    # https://github.com/bleachbit/bleachbit/issues/1444
+    # https://github.com/bleachbit/bleachbit/issues/1780
+    if os.environ.get('XDG_DATA_DIRS'):
+        return
+    xdg_data_dirs = [os.path.dirname(sys.executable) +
+                     '\\share',
+                     os.getcwd() + '\\share']
+
+    found_dir = False
+    for xdg_data_dir in xdg_data_dirs:
+        xdg_data_fn = os.path.join(
+            xdg_data_dir, 'glib-2.0', 'schemas', 'gschemas.compiled')
+        if os.path.exists(xdg_data_fn):
+            found_dir = True
+            break
+    if found_dir:
+        logger.debug('XDG_DATA_DIRS=%s', xdg_data_dir)
+        set_environ('XDG_DATA_DIRS', xdg_data_dir)
+    else:
+        logger.warning('XDG_DATA_DIRS not set and gschemas.compiled not found')
+
 
 def split_registry_key(full_key):
     r"""Given a key like HKLM\Software split into tuple (hive, key).
@@ -605,7 +701,7 @@ def symlink_or_copy(src, dst):
     try:
         os.symlink(src, dst)
         logger.debug('linked %s to %s', src, dst)
-    except (PermissionError, OSError) as e:
+    except (PermissionError, OSError):
         shutil.copy(src, dst)
         logger.debug('copied %s to %s', src, dst)
 
@@ -615,7 +711,8 @@ def has_fontconfig_cache(font_conf_file):
     fc_element = dom.getElementsByTagName('fontconfig')[0]
     cachefile = 'd031bbba323fd9e5b47e0ee5a0353f11-le32d8.cache-6'
     expanded_localdata = os.path.expandvars('%LOCALAPPDATA%')
-    expanded_homepath = os.path.join(os.path.expandvars('%HOMEDRIVE%'), os.path.expandvars('%HOMEPATH%'))
+    expanded_homepath = os.path.join(os.path.expandvars(
+        '%HOMEDRIVE%'), os.path.expandvars('%HOMEPATH%'))
     for dir_element in fc_element.getElementsByTagName('cachedir'):
 
         if dir_element.firstChild.nodeValue == 'LOCAL_APPDATA_FONTCONFIG_CACHE':
@@ -635,51 +732,64 @@ def has_fontconfig_cache(font_conf_file):
 
 
 def get_font_conf_file():
+    """Return the full path to fonts.conf"""
     if hasattr(sys, 'frozen'):
+        # running inside py2exe
         return os.path.join(bleachbit.bleachbit_exe_path, 'etc', 'fonts', 'fonts.conf')
 
     import gi
-    return os.path.join(
-            os.path.dirname(os.path.dirname(gi.__file__)),
-            'gnome', 'etc', 'fonts', 'fonts.conf'
-        )
+    gnome_dir = os.path.join(os.path.dirname(
+        os.path.dirname(gi.__file__)), 'gnome')
+    if not os.path.isdir(gnome_dir):
+        # BleachBit is running from a stand-alone Python installation.
+        gnome_dir = os.path.join(sys.exec_prefix, '..', '..')
+    return os.path.join(gnome_dir, 'etc', 'fonts', 'fonts.conf')
 
 
 class SplashThread(Thread):
     def __init__(self, group=None, target=None, name=None,
                  args=(), kwargs={}, Verbose=None):
         super().__init__(group, self._show_splash_screen, name, args, kwargs)
+        self._splash_screen_started = Event()
         self._splash_screen_handle = None
         self._splash_screen_height = None
         self._splash_screen_width = None
 
-    def run(self):
+    def start(self):
+        Thread.start(self)
+        self._splash_screen_started.wait()
         logger.debug('SplashThread started')
+
+    def run(self):
         self._splash_screen_handle = self._target()
+        self._splash_screen_started.set()
+
         # Dispatch messages
         win32gui.PumpMessages()
 
     def join(self, *args):
-        import win32con, win32gui
-        win32gui.PostMessage(self._splash_screen_handle, win32con.WM_CLOSE, 0, 0)
+        import win32con
+        import win32gui
+        win32gui.PostMessage(self._splash_screen_handle,
+                             win32con.WM_CLOSE, 0, 0)
         Thread.join(self, *args)
 
     def _show_splash_screen(self):
-        #get instance handle
+        # get instance handle
         hInstance = win32api.GetModuleHandle()
 
         # the class name
         className = 'SimpleWin32'
 
         # create and initialize window class
-        wndClass                = win32gui.WNDCLASS()
-        wndClass.style          = win32con.CS_HREDRAW | win32con.CS_VREDRAW
-        wndClass.lpfnWndProc    = self.wndProc
-        wndClass.hInstance      = hInstance
-        wndClass.hIcon          = win32gui.LoadIcon(0, win32con.IDI_APPLICATION)
-        wndClass.hCursor        = win32gui.LoadCursor(0, win32con.IDC_ARROW)
-        wndClass.hbrBackground  = win32gui.GetStockObject(win32con.WHITE_BRUSH)
-        wndClass.lpszClassName  = className
+        wndClass = win32gui.WNDCLASS()
+        wndClass.style = win32con.CS_HREDRAW | win32con.CS_VREDRAW
+        wndClass.lpfnWndProc = self.wndProc
+        wndClass.hInstance = hInstance
+        wndClass.hIcon = win32gui.LoadIcon(0, win32con.IDI_APPLICATION)
+        wndClass.hCursor = win32gui.LoadCursor(0, win32con.IDC_ARROW)
+        wndClass.hbrBackground = win32gui.GetStockObject(win32con.WHITE_BRUSH)
+        wndClass.lpszClassName = className
 
         # register window class
         wndClassAtom = None
@@ -696,7 +806,7 @@ class SplashThread(Thread):
         windowPosY = (displayHeigh - self._splash_screen_height) // 2
 
         hWindow = win32gui.CreateWindow(
-            wndClassAtom,                   #it seems message dispatching only works with the atom, not the class name
+            wndClassAtom,  # it seems message dispatching only works with the atom, not the class name
             'Bleachbit splash screen',
             win32con.WS_POPUPWINDOW |
             win32con.WS_VISIBLE,
@@ -734,7 +844,8 @@ class SplashThread(Thread):
         except Exception as e:
             exc_message = str(e)
             logger.debug(
-                'Failed attempt to show splash screen with keybd_event: {}'.format(exc_message)
+                'Failed attempt to show splash screen with keybd_event: {}'.format(
+                    exc_message)
             )
 
         if win32gui.GetForegroundWindow() == hWindow:
@@ -742,20 +853,24 @@ class SplashThread(Thread):
 
         # Solution 2: Attaching current thread to the foreground thread in order to use BringWindowToTop
         # https://shlomio.wordpress.com/2012/09/04/solved-setforegroundwindow-win32-api-not-always-works/
-        foreground_thread_id, foreground_process_id = win32process.GetWindowThreadProcessId(win32gui.GetForegroundWindow())
+        foreground_thread_id, _foreground_process_id = win32process.GetWindowThreadProcessId(
+            win32gui.GetForegroundWindow())
         appThread = win32api.GetCurrentThreadId()
 
         if foreground_thread_id != appThread:
 
             try:
-                win32process.AttachThreadInput(foreground_thread_id, appThread, True)
+                win32process.AttachThreadInput(
+                    foreground_thread_id, appThread, True)
                 win32gui.BringWindowToTop(hWindow)
                 win32gui.ShowWindow(hWindow, win32con.SW_SHOW)
-                win32process.AttachThreadInput(foreground_thread_id, appThread, False)
+                win32process.AttachThreadInput(
+                    foreground_thread_id, appThread, False)
             except Exception as e:
                 exc_message = str(e)
                 logger.debug(
-                    'Failed attempt to show splash screen with AttachThreadInput: {}'.format(exc_message)
+                    'Failed attempt to show splash screen with AttachThreadInput: {}'.format(
+                        exc_message)
                 )
 
         else:
@@ -766,17 +881,21 @@ class SplashThread(Thread):
             return True
 
         # Solution 3: Working with timers that lock/unlock SetForegroundWindow
-        #https://gist.github.com/EBNull/1419093
+        # https://gist.github.com/EBNull/1419093
         try:
-            timeout = win32gui.SystemParametersInfo(win32con.SPI_GETFOREGROUNDLOCKTIMEOUT)
-            win32gui.SystemParametersInfo(win32con.SPI_SETFOREGROUNDLOCKTIMEOUT, 0, win32con.SPIF_SENDCHANGE)
+            timeout = win32gui.SystemParametersInfo(
+                win32con.SPI_GETFOREGROUNDLOCKTIMEOUT)
+            win32gui.SystemParametersInfo(
+                win32con.SPI_SETFOREGROUNDLOCKTIMEOUT, 0, win32con.SPIF_SENDCHANGE)
             win32gui.BringWindowToTop(hWindow)
             win32gui.SetForegroundWindow(hWindow)
-            win32gui.SystemParametersInfo(win32con.SPI_SETFOREGROUNDLOCKTIMEOUT, timeout, win32con.SPIF_SENDCHANGE)
+            win32gui.SystemParametersInfo(
+                win32con.SPI_SETFOREGROUNDLOCKTIMEOUT, timeout, win32con.SPIF_SENDCHANGE)
         except Exception as e:
             exc_message = str(e)
             logger.debug(
-                'Failed attempt to show splash screen with SystemParametersInfo: {}'.format(exc_message)
+                'Failed attempt to show splash screen with SystemParametersInfo: {}'.format(
+                    exc_message)
             )
 
         if win32gui.GetForegroundWindow() == hWindow:
@@ -792,8 +911,10 @@ class SplashThread(Thread):
 
         if message == win32con.WM_PAINT:
             hDC, paintStruct = win32gui.BeginPaint(hWnd)
-            folder_with_ico_file = 'share' if hasattr(sys, 'frozen') else 'windows'
-            filename = os.path.join(os.path.dirname(sys.argv[0]), folder_with_ico_file, 'bleachbit.ico')
+            folder_with_ico_file = 'share' if hasattr(
+                sys, 'frozen') else 'windows'
+            filename = os.path.join(os.path.dirname(
+                sys.argv[0]), folder_with_ico_file, 'bleachbit.ico')
             flags = win32con.LR_LOADFROMFILE
             hIcon = win32gui.LoadImage(
                 0, filename, win32con.IMAGE_ICON,
@@ -801,14 +922,16 @@ class SplashThread(Thread):
 
             # Default icon size seems to be 32 pixels so we center the icon vertically.
             default_icon_size = 32
-            icon_top_margin = self._splash_screen_height - 2 * (default_icon_size + 2)
+            icon_top_margin = self._splash_screen_height - \
+                2 * (default_icon_size + 2)
             win32gui.DrawIcon(hDC, 0, icon_top_margin, hIcon)
             # win32gui.DrawIconEx(hDC, 0, 0, hIcon, 64, 64, 0, 0, win32con.DI_NORMAL)
 
             rect = win32gui.GetClientRect(hWnd)
             textmetrics = win32gui.GetTextMetrics(hDC)
             text_left_margin = 2 * default_icon_size
-            text_rect = (text_left_margin, (rect[3]-textmetrics['Height'])//2, rect[2], rect[3])
+            text_rect = (text_left_margin,
+                         (rect[3] - textmetrics['Height']) // 2, rect[2], rect[3])
             win32gui.DrawText(
                 hDC,
                 _("BleachBit is starting...\n"),
@@ -825,4 +948,5 @@ class SplashThread(Thread):
         else:
             return win32gui.DefWindowProc(hWnd, message, wParam, lParam)
 
+
 splash_thread = SplashThread()
