@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2024 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,37 +22,28 @@
 Code that is commonly shared throughout BleachBit
 """
 
-import gettext
-import locale
 import os
 import re
 import sys
-import platform
 
 from bleachbit import Log
-from configparser import RawConfigParser, NoOptionError # used in other files
+from configparser import RawConfigParser, NoOptionError  # used in other files
 
-APP_VERSION = "4.6.1"
+APP_VERSION = "5.0.0"
 APP_NAME = "BleachBit"
 APP_URL = "https://www.bleachbit.org"
+APP_COPYRIGHT = "Copyright (C) 2008-2025 Andrew Ziem"
 
 socket_timeout = 10
 
-if sys.version_info < (3,0,0):
-    print('BleachBit no longer supports Python 2.x.')
+if sys.version_info < (3, 8, 0):
+    print('BleachBit requires Python version 3.8 or later')
     sys.exit(1)
 
 if hasattr(sys, 'frozen') and sys.frozen == 'windows_exe':
     stdout_encoding = 'utf-8'
 else:
     stdout_encoding = sys.stdout.encoding
-    if 'win32' == sys.platform:
-        import win_unicode_console
-        win_unicode_console.enable()
-
-if not hasattr(platform, 'linux_distribution'):
-    from ._platform import _linux_distribution
-    platform.linux_distribution = _linux_distribution
 
 logger = Log.init_log()
 
@@ -107,7 +98,7 @@ try:
     options_dir = os.environ['BLEACHBIT_TEST_OPTIONS_DIR']
 except KeyError:
     pass
-        
+
 options_file = os.path.join(options_dir, "bleachbit.ini")
 
 # check whether the application is running from the source tree
@@ -130,7 +121,7 @@ personal_cleaners_dir = os.path.join(options_dir, "cleaners")
 # personal_cleaners_dir.
 if os.path.isdir(os.path.join(bleachbit_exe_path, 'cleaners')) and not portable_mode:
     system_cleaners_dir = os.path.join(bleachbit_exe_path, 'cleaners')
-elif sys.platform.startswith('linux') or sys.platform == 'darwin':
+elif sys.platform in ('linux', 'darwin'):
     system_cleaners_dir = '/usr/share/bleachbit/cleaners'
 elif sys.platform == 'win32':
     system_cleaners_dir = os.path.join(bleachbit_exe_path, 'share\\cleaners\\')
@@ -149,7 +140,8 @@ if portable_mode:
     local_cleaners_dir = os.path.join(bleachbit_exe_path, 'cleaners')
 
 # windows10 theme
-windows10_theme_path = os.path.normpath(os.path.join(bleachbit_exe_path, 'themes/windows10'))
+windows10_theme_path = os.path.normpath(
+    os.path.join(bleachbit_exe_path, 'themes/windows10'))
 
 # application icon
 __icons = (
@@ -182,7 +174,7 @@ if os.path.exists("./locale/"):
     # local locale (personal)
     locale_dir = os.path.abspath("./locale/")
 # system-wide installed locale
-elif sys.platform.startswith("linux") or sys.platform == "darwin":
+elif sys.platform in ('linux', 'darwin'):
     locale_dir = "/usr/share/locale/"
 elif sys.platform == "win32":
     locale_dir = os.path.join(bleachbit_exe_path, "share\\locale\\")
@@ -192,109 +184,12 @@ elif sys.platform.startswith("openbsd") or sys.platform.startswith("freebsd"):
     locale_dir = "/usr/local/share/locale/"
 
 
-
-#
-# gettext
-#
-try:
-    (user_locale, encoding) = locale.getdefaultlocale()
-except:
-    logger.exception('error getting locale')
-    user_locale = None
-    encoding = None
-
-if user_locale is None:
-    user_locale = 'C'
-    logger.warning("no default locale found.  Assuming '%s'", user_locale)
-
-if 'win32' == sys.platform:
-    os.environ['LANG'] = user_locale
-
-try:
-    if not os.path.exists(locale_dir):
-        raise RuntimeError('translations not installed')
-    t = gettext.translation('bleachbit', locale_dir)
-    _ = t.gettext
-except:
-    def _(msg):
-        """Dummy replacement for gettext"""
-        return msg
-
-try:
-    locale.bindtextdomain('bleachbit', locale_dir)
-except AttributeError:
-    if sys.platform.startswith('win'):
-        try:
-            # We're on Windows; try and use libintl-8.dll instead
-            import ctypes
-            libintl = ctypes.cdll.LoadLibrary('libintl-8.dll')
-        except OSError:
-            # libintl-8.dll isn't available; give up
-            pass
-        else:
-            # bindtextdomain can not handle Unicode
-            libintl.bindtextdomain(b'bleachbit', locale_dir.encode('utf-8'))
-            libintl.bind_textdomain_codeset(b'bleachbit', b'UTF-8')
-except:
-    logger.exception('error binding text domain')
-
-try:
-    ngettext = t.ngettext
-except:
-    def ngettext(singular, plural, n):
-        """Dummy replacement for plural gettext"""
-        if 1 == n:
-            return singular
-        return plural
-
-
-#
-# pgettext
-#
-
-# Code released in the Public Domain. You can do whatever you want with this package.
-# Originally written by Pierre Métras <pierre@alterna.tv> for the OLPC XO laptop.
-#
-# Original source: http://dev.laptop.org/git/activities/clock/plain/pgettext.py
-
-
-# pgettext(msgctxt, msgid) from gettext is not supported in Python as of January 2017
-# http://bugs.python.org/issue2504
-# This issue was fixed in Python 3.8, but we need to support older versions.
-# Meanwhile we get official support, we have to simulate it.
-# See http://www.gnu.org/software/gettext/manual/gettext.html#Ambiguities for
-# more information about pgettext.
-
-# The separator between message context and message id.This value is the same as
-# the one used in gettext.h, so PO files should be still valid when Python gettext
-# module will include pgettext() function.
-GETTEXT_CONTEXT_GLUE = "\004"
-
-
-def pgettext(msgctxt, msgid):
-    """A custom implementation of GNU pgettext().
-    """
-    if msgctxt is None or msgctxt == "":
-        return _(msgid)
-    translation = _(msgctxt + GETTEXT_CONTEXT_GLUE + msgid)
-    if translation.startswith(msgctxt + GETTEXT_CONTEXT_GLUE):
-        return msgid
-    else:
-        return translation
-
-
-# Map our pgettext() custom function to _p()
-_p = pgettext
-
-
 #
 # URLs
 #
 base_url = "https://update.bleachbit.org"
 help_contents_url = "%s/help/%s" \
     % (base_url, APP_VERSION)
-release_notes_url = "%s/release-notes/%s" \
-    % (base_url, APP_VERSION)
 update_check_url = "%s/update/%s" % (base_url, APP_VERSION)
 
 # set up environment variables
@@ -316,14 +211,12 @@ if 'posix' == os.name:
 # should be re.IGNORECASE on macOS
 fs_scan_re_flags = 0 if os.name == 'posix' else re.IGNORECASE
 
-#
-# Exceptions
-#
+if 'win32' == sys.platform:
+    import win32process
 
-
-# Python 3.6 is the first with this exception
-if hasattr(sys.modules['builtins'], 'ModuleNotFoundError'):
-    ModuleNotFoundError = sys.modules['builtins'].ModuleNotFoundError
-else:
-    class ModuleNotFoundError(Exception):
-        pass
+    for process in win32process.EnumProcessModules(-1):
+        name = win32process.GetModuleFileNameEx(-1, process)
+        if re.search(r'python\d+.dll$', name, re.IGNORECASE):
+            bindir = os.path.dirname(name)
+            os.environ['GDK_PIXBUF_MODULE_FILE'] = os.path.join(
+                bindir, 'lib', 'gdk-pixbuf-2.0', '2.10.0', 'loaders.cache')
